{{
BFdemo.spin
Propeller Spin/PASM demo of Urban Mller's Brainf*** language
Copyright (c) 2009 by John Steven Denson
See MIT license for terms
}}

{
This demo displays 99 bottles by default.
Other bf language demo code is available in the dat section below.
}

con                                  
  _CLKMODE = XTAL1+PLL16X       '
  _XINFREQ = 6_000_000          ' change these for your board and crystal

{{
BF is an 8 token language created by Urban Mller which is a variation of P" ... see http://en.wikipedia.org/wiki/Brainfuck
A BF program has an implicit byte pointer, called "the pointer", which is free to move around within an array of bytes,
initially all set to zero. The pointer itself is initialized to point to the beginning of the array.

The Brainf*** programming language consists of eight commands, each of which is represented as a single character.

>       Increment the pointer.
<       Decrement the pointer.
+       Increment the byte at the pointer.
-       Decrement the byte at the pointer.
.       Output the byte at the pointer.
,       Input a byte and store it in the byte at the pointer.
[       Jump forward past the matching ] if the byte at the pointer is zero.
]       Jump backward to the matching [ unless the byte at the pointer is zero.

The semantics of the BF commands can also be expressed in terms of C assuming that p has been previously defined as a char*:

>       becomes         ++p;
<       becomes         --p;
+       becomes         ++*p;
-       becomes         --*p;
.       becomes         putchar(*p);
,       becomes         *p = getchar();
[       becomes         while (*p) {
]       becomes         }
}}


con
{{
Variables for the program begin at VARSTART
}}

VARSTART = $7000
VARSIZE  = $8000-VARSTART
  
obj
  'bu : "BMAutility"
  bf : "BFinterp2"
  fd : "FullDuplexSerial"

var
  long progptr, varptr, inptr, outptr                   ' keep these in order
  long fdstack[$100]

dat
{{
In this demo, code can be entered in a dat section as a series of bytes or as a file.
Any code should be null terminated, thus, byte 0 should appear after program file "blah.bf"
}}

{{
This dat section represents a list of bf language demo code. 
There are some programs that do not work with the interpreter -- I'm looking at that.
}}

' print user char in p2 by p1 times. new line, then repeat  
  echocnt     byte "-["                                                         ' repeat 255 times (p0)
              'byte   ">++"                                                     ' int times to print in p1     
              byte   ">,"                                                       ' get chars to print number (0-9) in p1
              'byte   ">+++++++++++++++++++++++++++++++++"                      ' char to print in p2
              byte   ">++++++++[<------>-]<"                                    ' dirty atoi(c) decrement c by 8*6
              byte   ">,"                                                       ' char to print in p2
              byte   "<["                                                       ' print p2 p1 times
              byte     ">."                                                     ' print p2
              byte     "<-"                                                     ' decrement p1
              byte    "]"                                                       ' back to loop
              byte    ">[-]<"                                                   ' clear p2
              byte    ">++++++++++.+++.[-]<"                                    ' LF/CR then clear p2
              byte   "<]",0                                                     ' repeat
                
{
  umhello     byte "++++++++++[>+++++++>++++++++++>+++>+<<<<-]>++.>+.+++++++..+++.>++.<<+++++++++++++++.>.+++.------.--------.>+.>.",0 'um-hello.bf
  fromto0     byte ">++++ ++++[<++++++>-]<>++++++++++[<.+>-]+++++++++++[<.->-]",0 ' print 0123456789:9876543210
  brackets    byte "++++++++>,<[>.<-]",0  ' loop outer [ p1, clear p0
  putbang     byte "+++++++++++++++++++++++++++++++++.",0 ' send ! to console
  putchars    byte "+++++++++++++++++++++++++++++++++.++.+++.--.",0 ' send !#&$ to console
  repeatc     byte "+[.]",0
  echoone     byte ",.",0 ' input and echo key
  echoall     byte ",+[-.,+]",0 ' echos what is typed until char val == 255 (how do you do that?)
  putnums     byte "++++++++++++++++++++++++++++++++++++++++++++++++>++++++++++[<.+>-]",0
  dupvar      byte "++[>+>+<<-]>>[<<+>>-]<",0 ' duplicate var set p(n+1) = p(n); restore p(n) = p(n+2)
'}

  program     file "99bottles.bf"
  byte 0

  charset     byte ".+[.+]",0
  
pub main

  fd.start(31,30,0,115200)      ' start serial port for SPIN and BF
  waitcnt(clkfreq+cnt)
  
  fd.str(string($d,"Brainf***",$d))

  progptr := @program
  'progptr := @echocnt

  varptr  := VARSTART ' @vars
  longfill(varptr, 0, constant(VARSIZE>>2)) ' clear old junk

  cognew(fdhandler,@fdstack)
  waitcnt(clkfreq/10+cnt)
    
  bf.start(@progptr)

  repeat


pub fdhandler
'' handle IO requests from bf interpreter.
'' this could be replaced with other drivers.
  repeat
    if outptr
      fd.tx(outptr)
      outptr~
    if inptr
      inptr~ ' let interpreter know we're handling request 
      inptr := fd.rx  


{{
 TERMS OF USE: MIT License                                                           

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
  