{{
BFinterp2.spin
Propeller Spin/PASM version of Urban Mller's Brainf*** language interpreter
Copyright (c) 2009 by John Steven Denson
See MIT license for terms
}}

{
See BFdemo.spin for language description and examples.
The interpreter works with many bf programs, but there are some challenges.
}

obj
  'bu : "BMAutility"
  
var
  long cog
  
pub start(p)
  cog := cognew(@entry,p) + 1
  'cog := bu.debug(@entry, p)
  waitcnt(clkfreq/10+cnt)

pub stop
  if cog
    cogstop(cog~ - 1)


dat
entry         org
long    0 [8] ' BMA debugger stub

' setup parameters
'
              rdlong    pc,     par
              mov       ptr,    par
              add       ptr,    #4
              rdlong    ptr,    ptr
              mov       _inp,   par
              add       _inp,   #8
              mov       _outp,  par
              add       _outp,  #12

' jump over first program increment
'              
              jmp       #nxt    ' don't increment _prg first time
loop          add       pc,     #1
nxt           rdbyte    cmd,    pc   wz
        if_z  jmp       #$      ' freeze if zero

' with so few commands, a jump table doesn't make sense
'
              cmp       cmd,    #"+" wz         ' if + increment *ptr ... most common
       if_z   jmp       #doIncVal       
              cmp       cmd,    #"-" wz         ' if - decrement *ptr ... most common
       if_z   jmp       #doDecVal       
              cmp       cmd,    #">" wz         ' if > increment ptr  ... common
       if_z   jmp       #doIncPtr       
              cmp       cmd,    #"<" wz         ' if < decrement ptr  ... common
       if_z   jmp       #doDecPtr       
              cmp       cmd,    #"[" wz         ' if [ and *ptr do next command
       if_z   jmp       #doOpenLoop       
              cmp       cmd,    #"]" wz         ' if ] repeat while
       if_z   jmp       #doCloseLoop       
              cmp       cmd,    #"." wz         ' if . output val     ... less common
       if_z   jmp       #doOutput       
              cmp       cmd,    #"," wz         ' if , input to val   ... least common
       if_z   jmp       #doInput       
              jmp       #loop

'   cmp       val,    #">" wz         ' if > increment ptr
'
doIncPtr      add       ptr,    #1
              jmp       #loop

'   cmp       val,    #"<" wz         ' if < decrement ptr               
'
doDecPtr      sub       ptr,    #1
              jmp       #loop

'   cmp       val,    #"+" wz         ' if + increment *ptr               
'
doIncVal      rdbyte    val,    ptr
              add       val,    #1
              wrbyte    val,    ptr
              jmp       #loop
                                                
'   cmp       val,    #"-" wz         ' if - decrement *ptr               
'
doDecVal      rdbyte    val,    ptr
              sub       val,    #1
              wrbyte    val,    ptr
              jmp       #loop

'   cmp       val,    #"," wz         ' if , input to val
'
doInput       mov       tmp,    #$0f            ' some value
              wrbyte    tmp,    _inp            ' set non-zero val
              rdbyte    val,    _inp            ' wait for zero val
              tjnz      val,    #$-1
              rdbyte    val,    _inp            ' wait for non-zero val
              tjz       val,    #$-1
              wrbyte    val,    ptr
              mov       tmp,    #0
              wrbyte    tmp,    _inp
              jmp       #loop

'   cmp       val,    #"." wz         ' if . output val
'
doOutput      rdbyte    val,    ptr
              mov       parm,   val
              call      #putch
              jmp       #loop

putch         wrbyte    parm,    _outp           ' outp handler waits for non-zero val
              rdbyte    parm,    _outp           ' get val until zero ...
              tjnz      parm,    #$-1            ' if zero, we're done
putch_ret     ret

'   cmp       val,     #"[" wz         ' if [ and *ptr do next command
'
doOpenLoop    add       brk,    #1              ' in bracket loop              
              rdbyte    val,    ptr
              tjnz      val,    #loop           ' if <> 0, do commands
              sub       brk,    #1              ' not in bracket loop
              mov       bct,    #0
skip          add       pc,     #1
              rdbyte    cmd,    pc  
              cmp       cmd,    #"[" wz         ' if [ incr local bracket cnt bct
        if_e  add       bct,    #1
              cmp       cmd,    #"]" wz         ' if ] repeat while
        if_ne jmp       #skip
              cmp       bct,    #0 wz           ' if local bracket cnt <> 0, more to skip
        if_e  jmp       #loop                   ' already at next pc?
              sub       bct,    #1
              'mov       parm,   cmd
              'call      #putch
              jmp       #skip

'   cmp       val,    #"]" wz         ' if ] repeat while
doCloseLoop   cmp       brk,    #0   wz         ' in bracket statement?
        if_z  jmp       #loop                   ' none. fetch next instruction
              sub       brk,    #1              ' next ?
              rdbyte    val,    ptr             ' get condition
              tjz       val,    #loop           ' if val == 0, fetch next instruction
              mov       bct,    #0              
back          sub       pc,     #1              ' if val <> 0, search for beginning
              rdbyte    cmd,    pc
              cmp       cmd,    #"]" wz         ' if ] incr local bracket cnt bct
        if_e  add       bct,    #1
              cmp       cmd,    #"[" wz         ' go back to [
        if_ne jmp       #back
              cmp       bct,    #0 wz           ' if local bracket cnt <> 0, more to skip
        if_e  jmp       #nxt                    ' jmp next since we already have '['
              sub       bct,    #1
              jmp       #back


pc            long 0    ' nop   - program counter
val           long 0    ' nop   - space for value
ptr           long 0    ' nop   - pointer to value

brk           long 0    ' nop   - bracket count
bct           long 0    ' nop   - local bracket count

cmd           long 0    ' nop   - space for instruction ASCII cmd char
parm          long 0    ' nop   - some parameter
tmp           long 0    ' nop   - some temporary value
tcnt          long 0    ' nop   - some temporary count
_inp          long 0    ' nop   - input address
_outp         long 0    ' nop   - output address

{{
 TERMS OF USE: MIT License                                                           

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
         